'use strict';

exports.__esModule = true;

var _SystemRenderer2 = require('../SystemRenderer');

var _SystemRenderer3 = _interopRequireDefault(_SystemRenderer2);

var _utils = require('../../utils');

var _MaskSystem = require('./systems/MaskSystem');

var _MaskSystem2 = _interopRequireDefault(_MaskSystem);

var _StencilSystem = require('./systems/StencilSystem');

var _StencilSystem2 = _interopRequireDefault(_StencilSystem);

var _FilterSystem = require('./systems/FilterSystem');

var _FilterSystem2 = _interopRequireDefault(_FilterSystem);

var _FramebufferSystem = require('./systems/FramebufferSystem');

var _FramebufferSystem2 = _interopRequireDefault(_FramebufferSystem);

var _RenderTextureSystem = require('./systems/RenderTextureSystem');

var _RenderTextureSystem2 = _interopRequireDefault(_RenderTextureSystem);

var _TextureSystem = require('./systems/textures/TextureSystem');

var _TextureSystem2 = _interopRequireDefault(_TextureSystem);

var _ProjectionSystem = require('./systems/ProjectionSystem');

var _ProjectionSystem2 = _interopRequireDefault(_ProjectionSystem);

var _StateSystem = require('./systems/StateSystem');

var _StateSystem2 = _interopRequireDefault(_StateSystem);

var _GeometrySystem = require('./systems/geometry/GeometrySystem');

var _GeometrySystem2 = _interopRequireDefault(_GeometrySystem);

var _ShaderSystem = require('./systems/shader/ShaderSystem');

var _ShaderSystem2 = _interopRequireDefault(_ShaderSystem);

var _ContextSystem = require('./systems/ContextSystem');

var _ContextSystem2 = _interopRequireDefault(_ContextSystem);

var _BatchSystem = require('./systems/BatchSystem');

var _BatchSystem2 = _interopRequireDefault(_BatchSystem);

var _const = require('../../const');

var _UniformGroup = require('../../shader/UniformGroup');

var _UniformGroup2 = _interopRequireDefault(_UniformGroup);

var _math = require('../../math');

var _miniRunner = require('mini-runner');

var _miniRunner2 = _interopRequireDefault(_miniRunner);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }
// import TextureGCSystem from './systems/textures/TextureGCSystem';


/**
 * The WebGLRenderer draws the scene and all its content onto a webGL enabled canvas. This renderer
 * should be used for browsers that support webGL. This Render works by automatically managing webGLBatchs.
 * So no need for Sprite Batches or Sprite Clouds.
 * Don't forget to add the view to your DOM or you will not see anything :)
 *
 * @class
 * @memberof PIXI
 * @extends PIXI.SystemRenderer
 */
var WebGLRenderer = function (_SystemRenderer) {
    _inherits(WebGLRenderer, _SystemRenderer);

    // eslint-disable-next-line valid-jsdoc
    /**
     *
     * @param {object} [options] - The optional renderer parameters
     * @param {number} [options.width=800] - the width of the screen
     * @param {number} [options.height=600] - the height of the screen
     * @param {HTMLCanvasElement} [options.view] - the canvas to use as a view, optional
     * @param {boolean} [options.transparent=false] - If the render view is transparent, default false
     * @param {boolean} [options.autoResize=false] - If the render view is automatically resized, default false
     * @param {boolean} [options.antialias=false] - sets antialias. If not available natively then FXAA
     *  antialiasing is used
     * @param {boolean} [options.forceFXAA=false] - forces FXAA antialiasing to be used over native.
     *  FXAA is faster, but may not always look as great
     * @param {number} [options.resolution=1] - The resolution / device pixel ratio of the renderer.
     *  The resolution of the renderer retina would be 2.
     * @param {boolean} [options.clearBeforeRender=true] - This sets if the CanvasRenderer will clear
     *  the canvas or not before the new render pass. If you wish to set this to false, you *must* set
     *  preserveDrawingBuffer to `true`.
     * @param {boolean} [options.preserveDrawingBuffer=false] - enables drawing buffer preservation,
     *  enable this if you need to call toDataUrl on the webgl context.
     * @param {boolean} [options.roundPixels=false] - If true Pixi will Math.floor() x/y values when
     *  rendering, stopping pixel interpolation.
     * @param {boolean} [options.legacy=false] - If true Pixi will aim to ensure compatibility
     * with older / less advanced devices. If you experiance unexplained flickering try setting this to true.
     */
    function WebGLRenderer(options, arg2, arg3) {
        _classCallCheck(this, WebGLRenderer);

        /**
         * The type of this renderer as a standardised const
         *
         * @member {number}
         * @see PIXI.RENDERER_TYPE
         */
        var _this = _possibleConstructorReturn(this, _SystemRenderer.call(this, 'WebGL', options, arg2, arg3));

        _this.type = _const.RENDERER_TYPE.WEBGL;

        // this will be set by the contextSystem (this.context)
        _this.gl = null;
        _this.CONTEXT_UID = 0;
        _this.legacy = !!options.legacy;

        // TODO legacy!

        // runners!
        _this.runners = {
            destroy: new _miniRunner2.default('destroy'),
            contextChange: new _miniRunner2.default('contextChange', 1),
            reset: new _miniRunner2.default('reset'),
            update: new _miniRunner2.default('update'),
            postrender: new _miniRunner2.default('postrender'),
            prerender: new _miniRunner2.default('prerender'),
            resize: new _miniRunner2.default('resize', 2)
        };

        _this._backgroundColorRgba[3] = _this.transparent ? 0 : 1;

        _this.globalUniforms = new _UniformGroup2.default({
            projectionMatrix: new _math.Matrix()
        }, true);

        _this.addSystem(_MaskSystem2.default, 'mask').addSystem(_ContextSystem2.default, 'context').addSystem(_StateSystem2.default, 'state').addSystem(_ShaderSystem2.default, 'shader').addSystem(_TextureSystem2.default, 'texture').addSystem(_GeometrySystem2.default, 'geometry').addSystem(_FramebufferSystem2.default, 'framebuffer').addSystem(_StencilSystem2.default, 'stencil').addSystem(_ProjectionSystem2.default, 'projection'
        // .addSystem(TextureGCSystem)
        ).addSystem(_FilterSystem2.default, 'filter').addSystem(_RenderTextureSystem2.default, 'renderTexture').addSystem(_BatchSystem2.default, 'batch');

        _this.initPlugins();

        /**
         * The options passed in to create a new webgl context.
         *
         * @member {object}
         * @private
         */
        if (options.context) {
            _this.context.initFromContext(options.context);
        } else {
            _this.context.initFromOptions({
                alpha: _this.transparent,
                antialias: options.antialias,
                premultipliedAlpha: _this.transparent && _this.transparent !== 'notMultiplied',
                stencil: true,
                preserveDrawingBuffer: options.preserveDrawingBuffer
            });
        }

        _this.renderingToScreen = true;

        _this._initContext();

        (0, _utils.sayHello)(_this.context.webGLVersion === 2 ? 'WebGL 2' : 'WebGL 1');
        return _this;
    }

    WebGLRenderer.prototype.addSystem = function addSystem(_class, name) {
        if (!name) {
            name = _class.name;
        }

        // TODO - read name from class.name..

        /*
        if(name.includes('System'))
        {
            name = name.replace('System', '');
            name = name.charAt(0).toLowerCase() + name.slice(1);
        }
        */

        var system = new _class(this);

        if (this[name]) {
            throw new Error('Whoops! ' + name + ' is already a manger');
        }

        this[name] = system;

        for (var i in this.runners) {
            this.runners[i].add(system);
        }

        return this;

        /**
         * Fired after rendering finishes.
         *
         * @event PIXI.WebGLRenderer#postrender
         */

        /**
         * Fired before rendering starts.
         *
         * @event PIXI.WebGLRenderer#prerender
         */

        /**
         * Fired when the WebGL context is set.
         *
         * @event PIXI.WebGLRenderer#context
         * @param {WebGLRenderingContext} gl - WebGL context.
         */
    };

    /**
     * Creates the WebGL context
     *
     * @private
     */


    WebGLRenderer.prototype._initContext = function _initContext() {
        var gl = this.gl;

        var maxTextures = gl.getParameter(gl.MAX_TEXTURE_IMAGE_UNITS);

        this.boundTextures = new Array(maxTextures);
        this.emptyTextures = new Array(maxTextures);

        var tempObj = { _glTextures: {} };

        for (var i = 0; i < maxTextures; i++) {
            this.boundTextures[i] = tempObj;
        }

        // setup the width/height properties and gl viewport
        this.resize(this.screen.width, this.screen.height);
    };

    /**
     * Renders the object to its webGL view
     *
     * @param {PIXI.DisplayObject} displayObject - the object to be rendered
     * @param {PIXI.RenderTexture} renderTexture - The render texture to render to.
     * @param {boolean} [clear] - Should the canvas be cleared before the new render
     * @param {PIXI.Transform} [transform] - A transform to apply to the render texture before rendering.
     * @param {boolean} [skipUpdateTransform] - Should we skip the update transform pass?
     */


    WebGLRenderer.prototype.render = function render(displayObject, renderTexture, clear, transform, skipUpdateTransform) {
        // can be handy to know!
        this.renderingToScreen = !renderTexture;

        this.runners.prerender.run();
        this.emit('prerender');

        // no point rendering if our context has been blown up!
        if (this.context.isLost) {
            return;
        }

        if (!renderTexture) {
            this._lastObjectRendered = displayObject;
        }

        if (!skipUpdateTransform) {
            // update the scene graph
            var cacheParent = displayObject.parent;

            displayObject.parent = this._tempDisplayObjectParent;
            displayObject.updateTransform();
            displayObject.parent = cacheParent;
            // displayObject.hitArea = //TODO add a temp hit area
        }

        this.renderTexture.bind(renderTexture);
        this.batch.currentRenderer.start();

        if (clear !== undefined ? clear : this.clearBeforeRender) {
            this.renderTexture.clear();
        }

        displayObject.renderWebGL(this);

        // apply transform..
        this.batch.currentRenderer.flush();

        this.runners.postrender.run();
        this.emit('postrender');
    };

    /**
     * Resizes the webGL view to the specified width and height.
     *
     * @param {number} screenWidth - the new width of the screen
     * @param {number} screenHeight - the new height of the screen
     */


    WebGLRenderer.prototype.resize = function resize(screenWidth, screenHeight) {
        _SystemRenderer3.default.prototype.resize.call(this, screenWidth, screenHeight);
        this.runners.resize.run(screenWidth, screenHeight);
    };

    /**
     * Resets the WebGL state so you can render things however you fancy!
     *
     * @return {PIXI.WebGLRenderer} Returns itself.
     */


    WebGLRenderer.prototype.reset = function reset() {
        this.runners.reset.run();

        return this;
    };

    /**
     * Removes everything from the renderer (event listeners, spritebatch, etc...)
     *
     * @param {boolean} [removeView=false] - Removes the Canvas element from the DOM.
     *  See: https://github.com/pixijs/pixi.js/issues/2233
     */


    WebGLRenderer.prototype.destroy = function destroy(removeView) {
        this.runners.destroy.run();

        // call base destroy
        _SystemRenderer.prototype.destroy.call(this, removeView);

        this.destroyPlugins();

        // TODO nullify all the managers..
        this.gl = null;
    };

    return WebGLRenderer;
}(_SystemRenderer3.default);

/**
 * Collection of installed plugins. These are included by default in PIXI, but can be excluded
 * by creating a custom build. Consult the README for more information about creating custom
 * builds and excluding plugins.
 * @name PIXI.WebGLRenderer#plugins
 * @type {object}
 * @readonly
 * @property {PIXI.accessibility.AccessibilityManager} accessibility Support tabbing interactive elements.
 * @property {PIXI.extract.WebGLExtract} extract Extract image data from renderer.
 * @property {PIXI.interaction.InteractionManager} interaction Handles mouse, touch and pointer events.
 * @property {PIXI.prepare.WebGLPrepare} prepare Pre-render display objects.
 */

/**
 * Adds a plugin to the renderer.
 *
 * @method PIXI.WebGLRenderer#registerPlugin
 * @param {string} pluginName - The name of the plugin.
 * @param {Function} ctor - The constructor function or class for the plugin.
 */

exports.default = WebGLRenderer;
_utils.pluginTarget.mixin(WebGLRenderer);
//# sourceMappingURL=WebGLRenderer.js.map